/*
 * $Id: tosh_hci.h,v 1.3 2003/07/29 01:43:38 cws Exp $
 * Copyright (C) 2003 Chatchawan Wongsiriprasert
 *
 * Based on toshset(http://www.schwieters.org/toshset/) by Charles D. Schwieters
 * Based on tools(http://www.buzzard.org.uk/toshiba/) by Jonathan Buzzard
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
#ifndef __TOSH_HCI_H__
#define __TOSH_HCI_H__

#include "tosh_vars.h"
#include <map>

CToshVar* initToshHCI();

class CToshHCIVar: public CToshVar
{
public:
    enum HCI_CONSTANT
    {
        HCI_READ = 0xFE00,
        HCI_WRITE = 0xFF00,
    };
    
    enum HCI_FUNCTION
    {
        HCI_FUNC_NONE       = 0xFFFF,
        HCI_BACKLIGHT       = 0x0002,
        HCI_AC_ADAPTOR      = 0x0003,
        HCI_FAN             = 0x0004,
        HCI_SOFTWARE_SUSPEND = 0x0010,
        HCI_FLAT_PANEL       = 0x0011,
        HCI_SELECT_STATUS    = 0x0014,
        HCI_SYSTEM_EVENT     = 0x0016,
        HCI_FIR_STATUS       = 0x001b,
        HCI_VIDEO_OUT        = 0x001c,
        HCI_HOTKEY_EVENT     = 0x001e,
        HCI_UNUSED_MEMORY    = 0x0021,
        HCI_LOCK_STATUS      = 0x0022,
        HCI_BOOT_DEVICE      = 0x0026,
        HCI_OWNERSTRING      = 0x0029,
        HCI_LCD_BRIGNTNESS = 0x002a,
        HCI_HIBERNATION_INFO = 0x002d,
        HCI_HIBERNATION_LBA  = 0x002e,
        HCI_CPU_SPEED		=	0x0032,
        HCI_POWER_UP         = 0x0048
    };

    // HCI error codes
    enum HCI_ERROR {
        HCI_SUCCESS         = 0x00,
        HCI_FAILURE         = 0x01,
        HCI_NOT_SUPPORTED   = 0x80,
        HCI_INPUT_ERROR     = 0x83,
        HCI_WRITE_PROTECTED = 0x84,
        HCI_FIFO_EMPTY      = 0x8c,
        HCI_NOTREADY        = 0xff
    };

    static bool hci_call(unsigned& result,unsigned eax,unsigned ebx,
                                              unsigned ecx,unsigned edx) throw();
    static inline bool hci_read(unsigned& result,HCI_FUNCTION fn,
                                              unsigned ecx,unsigned edx) throw();
    static inline bool hci_write(unsigned& result,HCI_FUNCTION fn,
                                               unsigned ecx,unsigned edx) throw();
                                               
    static unsigned hci_read(HCI_FUNCTION fn,unsigned ecx,unsigned edx) 
                                                throw(std::runtime_error);
    static void hci_write(HCI_FUNCTION fn,unsigned ecx,unsigned edx) 
                                                throw(std::runtime_error);                                                
protected:                        
    typedef std::pair<unsigned,std::string> VAR_INFO;
    
    HCI_FUNCTION m_FunctionID;
    std::map<unsigned,VAR_INFO> m_Values;
    unsigned m_Mask;
    unsigned m_Shift;
    
public:
    CToshHCIVar(HCI_FUNCTION id,unsigned shift = 0,unsigned mask = 0xFFFFFFFF) 
    : CToshVar(),m_FunctionID(id),m_Values(),m_Mask(mask),m_Shift(shift)
    {}
    virtual ~CToshHCIVar()
    {}
    
    unsigned hci_read(unsigned ecx,unsigned edx) const
    { return hci_read(m_FunctionID,ecx,edx); }
    void hci_write(unsigned ecx,unsigned edx) 
    { hci_write(m_FunctionID,ecx,edx); }
    
    virtual std::string read(const std::string& item_name,
                                          unsigned fmt) const;
    virtual void write(const std::string& item_name,
                                 const std::string& value,unsigned fmt);
                                
    virtual void getItems(std::vector<std::string>& item_names) const;
    
    
    static std::string getErrorMsg(unsigned code);
    
    void addValue(unsigned value,unsigned show_value,
                          const std::string& str_value);
};

class CReadOnlyToshHCIVar: public CToshHCIVar
{
public:
    CReadOnlyToshHCIVar(HCI_FUNCTION id,
                                    unsigned shift = 0,unsigned mask = 0xFFFFFFFF) 
    : CToshHCIVar(id,shift,mask)
    {}
    virtual ~CReadOnlyToshHCIVar()
    {}
    
    virtual void write(const std::string& item_name,
                                const std::string& value,unsigned fmt)
    {
        throwUnsupportWrite();
    }
};

class CWriteOnlyToshHCIVar: public CToshHCIVar
{
public:
    CWriteOnlyToshHCIVar(HCI_FUNCTION id,
                                    unsigned shift = 0,unsigned mask = 0xFFFFFFFF) 
    : CToshHCIVar(id,shift,mask) 
    {}
    virtual ~CWriteOnlyToshHCIVar()
    {}
    
    virtual std::string read(const std::string& item_name,
                                          unsigned fmt) const
    {
        throwUnsupportRead(); return std::string(); 
    }
};

//---------------------------------------------------
// Inline functions
inline bool
CToshHCIVar::hci_read(unsigned& result,HCI_FUNCTION fn,
                              unsigned ecx,unsigned edx) throw()
{
    return hci_call(result,HCI_READ,fn,ecx,edx);
}

inline bool
CToshHCIVar::hci_write(unsigned& result,HCI_FUNCTION fn,
                              unsigned ecx,unsigned edx) throw()
{
    return hci_call(result,HCI_WRITE,fn,ecx,edx);
}
#endif
