/*
 * Copyright (C) 2003 Chatchawan Wongsiriprasert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
static const char PROGRAM_ID[]="$Id: toshctl.cpp,v 1.5 2003/03/20 02:25:04 cws Exp $";

#include "toshctl.h"
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <libgen.h>
#include <iostream>

#include "tosh_vars.h"
#include "tosh_hci.h"
#include "tosh_utils.h"

CToshVar* initToshBiosInfo();
CToshVar* initToshHCI();
                                         
CToshCtlApp::CToshCtlApp(int argc,char* argv[])
:m_VarMap(),m_Argc(argc),m_Argv(argv),
m_OptShowAll(false),m_OptShowBin(false),
m_OptShowName(true),m_OptShowValue(true),
m_OptSepChar(": "),m_OptShowFmt(CToshVar::FMT_STD),
m_euid(geteuid())
{
    seteuid(getuid());
    if ( argc > 0 )
    {
        m_AppName = basename(argv[0]);
    }
}

CToshCtlApp::~CToshCtlApp()
{
    TOSHVAR_MAP::iterator i;
    for(i=m_VarMap.begin();i != m_VarMap.end(); ++i)
    {
            delete i->second;
    }
}

void 
CToshCtlApp::usage()
{
    std::cerr << "usage: " << m_AppName <<  "  [-beNnr] variable[=value] ...\n";
    std::cerr << "       " << m_AppName <<  "  [-beNnr] -a\n";
    std::cerr << "  -a List all variables\n";
    std::cerr << "  -b Show only variables without any seperator and new line\n";
    std::cerr << "  -e Separate the name and the value of the variable(s) with `='\n";
    std::cerr << "  -N Show only variable names\n";
    std::cerr << "  -n Show only variable values\n";
    std::cerr << "  -m Show variable values in human readable format\n";
    std::cerr << "  -r Show/Set variable in raw (as read from hardware) format\n";
    std::cerr << "  -v Show version and exit\n";
}

void 
CToshCtlApp::showVersion()
{
    char version_string[] = "$Name: r0_0_0p3 $";
    char * start = version_string;

    while ( *start != 0 && *start != ':' ) ++start; 
    if ( *start == ':' ) ++start;

    while ( *start == ' ' ) ++start;
    char *end = start;
    while ( *end != ' ' && *end != 0 && *end != '$' ) 
    {
	if ( *end == '_' ) *end = '.';
	++end;
    }
    *end = 0;
	
    std::cout << m_AppName << ' ' << start << '\n';
}

int 
CToshCtlApp::run()
{
    switch ( parseCmdLine() ) 
    {
	case 1:
        	usage();
        	return 1;
	case 2:
		showVersion();
		return 0;
    }
    std::string msg = initTosh();

    if ( !msg.empty() )
    {
        std::cerr << m_AppName <<  ": " <<  msg << "\n";
        return 3;
    }
    
    if ( m_OptShowAll )
    {
        TOSHVAR_MAP::const_iterator i;
        for(i=m_VarMap.begin();i != m_VarMap.end(); ++i)
        {
            showToshVar(i->first,std::string(),i->second);
        }
        
        return 0;
    }
    
    for(int i = 0; i < m_Argc; ++i)
    {
        std::string var_name,item_name,var_value;
        int ret = parseVar(var_name,item_name,var_value,m_Argv[i]);
        
        if ( ret < 0)
        {
            std::cerr <<   m_AppName <<  " : invalid format " << m_Argv[i] << "\n";
            continue;
        }
        
        TOSHVAR_MAP::iterator it = m_VarMap.find(var_name);
        if ( it == m_VarMap.end() )
        {
            std::cerr << m_AppName << ": unknown oid '" 
                            << var_name << '.' << item_name << "'\n";
            continue;
        }
        
        if  ( ret == 0 )
        {
            showToshVar(var_name,item_name,it->second);
            continue;
        }
        
        if ( ret == 1 )
        {
            setToshVar(var_name,item_name,it->second,var_value);
        }
    }
    
    return 0;
}

int 
CToshCtlApp::parseCmdLine()
{
    if ( m_Argc < 2 ) return 1;
    int ch;

    while ((ch = getopt(m_Argc, m_Argv, "abemNnrv")) != -1) 
    {
        switch(ch) {
            case  'a':
                    m_OptShowAll = true;
                    break;
            case 'b':
                    m_OptShowBin = true;
                    break;
            case 'e':
                    m_OptSepChar ='=';
                    break;
             case 'm':
                    m_OptShowFmt = CToshVar::FMT_HUMAN;
                    break;
            case 'N':
                    m_OptShowValue = false;
                    break;
            case 'n':
                    m_OptShowName = false;
                    break;
            case 'r':
                    m_OptShowFmt = CToshVar::FMT_RAW;
                    break;
	    case 'v':
		    return 2;
            default:
                    return 1;
        }
    }
    
    m_Argc -= optind;
    m_Argv += optind;
       
    return 0;
}
    
int 
CToshCtlApp::parseVar(std::string& var_name,std::string& item_name,
                                  std::string& var_value,const char* param) const
{
    var_name.erase();
    item_name.erase();
    var_value.erase();
    
    if ( param == 0 ) return -1;
    
    const char *dot_pos = strchr(param,'.');
    const char *equal_pos = strchr(param,'=');
    const char *end = param + strlen(param);
    
    if ( equal_pos == 0 ) equal_pos = end;
    if ( dot_pos == 0 || dot_pos > equal_pos ) dot_pos = equal_pos;
    
    var_name.assign(param,dot_pos - param);
    ++dot_pos;
    if ( dot_pos < equal_pos  ) item_name.assign(dot_pos,equal_pos-dot_pos);
    if ( equal_pos == end ) return 0;
    
    ++equal_pos;
    var_value.assign(equal_pos,end-equal_pos);
    
    return 1;
}

void 
CToshCtlApp::showToshVar(const std::string& var_name,
                                        const std::string& item_name,
                                        const CToshVar* var) const
{
    std::vector<std::string> item_names;
    
    try
    {
        var->getItems(item_names,item_name);
        
        if  ( item_names.empty() )
        {
            item_names.push_back(item_name);
        }
                        
        std::vector<std::string>::const_iterator j;
        for(j = item_names.begin();j != item_names.end(); ++j)
        {
            std::string s = var->read(*j,m_OptShowFmt);
            formatOutput(var_name,*j,s);
        }
    }
    catch(const std::exception& exp)
    {
        std::cerr << var_name << '.' <<  item_name  
                       << ": " << exp.what() << "\n";
        return;
    }
    catch(...)
    {
        std::cerr << var_name << '.' <<  item_name  
                       << ": (read) Unknown error\n";
        return;
    }
}

void 
CToshCtlApp::setToshVar(const std::string& var_name,
                          const std::string& item_name,
                          CToshVar* var,
                          const std::string& value)                          
{
    std::string v;
    try
    {
         v = var->read(item_name,m_OptShowFmt);
         
         unsigned fmt;
         if ( m_OptShowFmt == CToshVar::FMT_HUMAN ) fmt = CToshVar::FMT_STD;
         else fmt = m_OptShowFmt;
         
         var->write(item_name,value,fmt);
         v += " -> ";
         v += var->read(item_name,m_OptShowFmt);
         
         formatOutput(var_name,item_name,v);         
    }
    catch(const std::exception& exp)
    {
        std::cerr << var_name << '.' <<  item_name 
                       << ": " << exp.what() << "\n";
        return;
    }
    catch(...)
    {
        std::cerr << var_name << '.' <<  item_name  
                      << ": (write) Unknown error\n";
        return;
    }
}

void 
CToshCtlApp::formatOutput(const std::string& var_name,
                                         const std::string& item_name,
                                         const std::string& value) const
{
   if ( m_OptShowBin )
    {
        std::cout << value;
        return;
    }

    if ( m_OptShowName )
    {
        std::cout << var_name;
        if ( !item_name.empty() ) std::cout << '.' << item_name;
    }
    
    if ( m_OptShowValue) 
    {
	if ( m_OptShowName ) std::cout << m_OptSepChar;
        std::cout << value;
    }
    
    std::cout << "\n";
}
                            
class CSetEUID
{
    int m_euid;
public:
    CSetEUID(int euid) : m_euid(euid) {}
    ~CSetEUID() { unset(); }

    void set() { seteuid(m_euid); }
    void unset() { seteuid(getuid()); }
};

std::string 
CToshCtlApp::initTosh()
{
    static const std::string msg_unknown_error("Unknown error");
    static const std::string msg_need_root("root privileges needed");
    
    CSetEUID euid(m_euid);

    // Get direct IO permission for this process
    // Need not to close.
    euid.set();
    if ( open ("/dev/io", O_RDONLY) < 0 )  
    {
        return msg_need_root;
    }
    
    try
    {
	// Only BIOS init code need root access to /dev/mem
        m_VarMap["bios"] = initToshBiosInfo();
	euid.unset();

        m_VarMap["hci"] = initToshHCI();
       // m_VarMap["sci"] = new CToshSCI();
    }
    catch(const char* str)
    {
        return str;
    }
    catch(std::exception& exp)
    {
        return exp.what();
    }
    catch(...)
    {
        return msg_unknown_error;
    }
    
    return std::string();
}
